@@ -1,7 +1,7 @@
 /*
  * Image handling routines for HTMLDOC, a HTML document processing program.
  *
- * Copyright © 2011-2021 by Michael R Sweet.
+ * Copyright © 2011-2022 by Michael R Sweet.
  * Copyright © 1997-2010 by Easy Software Products.  All rights reserved.
  *
  * This program is free software.  Distribution and use rights are outlined in
@@ -13,6 +13,7 @@
  */
 
 #include "htmldoc.h"
+#include <setjmp.h>
 
 #ifdef HAVE_LIBJPEG
 extern "C" {		/* Workaround for JPEG header problems... */
@@ -25,6 +26,13 @@ extern "C" {		/* Workaround for JPEG header problems... */
 #endif // HAVE_LIBPNG
 
 
+/*
+ * Limits...
+ */
+
+#define IMAGE_MAX_DIM	37837		// Maximum dimension - sqrt(4GiB / 3)
+
+
 /*
  * GIF definitions...
  */
@@ -224,8 +232,7 @@ gif_get_code(FILE *fp,		/* I - File to read from */
 
     if (done)
     {
-      progress_error(HD_ERROR_READ_ERROR,
-                     "Not enough data left to read GIF compression code.");
+      progress_error(HD_ERROR_READ_ERROR, "Not enough data left to read GIF compression code.");
       return (-1);	/* Sorry, no more... */
     }
 
@@ -249,7 +256,7 @@ gif_get_code(FILE *fp,		/* I - File to read from */
     * Read in another buffer...
     */
 
-    if ((count = gif_get_block (fp, buf + last_byte)) <= 0)
+    if ((count = gif_get_block(fp, buf + last_byte)) <= 0)
     {
      /*
       * Whoops, no more data!
@@ -263,7 +270,7 @@ gif_get_code(FILE *fp,		/* I - File to read from */
     * Update buffer state...
     */
 
-    curbit    = (curbit - lastbit) + 8 * last_byte;
+    curbit    = curbit + 8 * last_byte - lastbit;
     last_byte += (unsigned)count;
     lastbit   = last_byte * 8;
   }
@@ -305,12 +312,18 @@ gif_read_image(FILE       *fp,		/* I - Input file */
   pass      = 0;
   code_size = (uchar)getc(fp);
 
+  if (code_size > 12)
+  {
+    progress_error(HD_ERROR_READ_ERROR, "Bad GIF file \"%s\" - invalid code size %d.", img->filename, code_size);
+    return (-1);
+  }
+
   if (gif_read_lzw(fp, 1, code_size) < 0)
     return (-1);
 
   temp = img->pixels;
 
-  while ((pixel = gif_read_lzw(fp, 0, code_size)) >= 0)
+  while ((pixel = gif_read_lzw(fp, 0, code_size)) >= 0 && pixel < 256)
   {
     temp[0] = cmap[pixel][0];
 
@@ -432,7 +445,7 @@ gif_read_lzw(FILE *fp,			/* I - File to read from */
   if (sp > stack)
     return (*--sp);
 
-  while ((code = gif_get_code (fp, code_size, 0)) >= 0)
+  while ((code = gif_get_code(fp, code_size, 0)) >= 0)
   {
     if (code == clear_code)
     {
@@ -459,7 +472,6 @@ gif_read_lzw(FILE *fp,			/* I - File to read from */
     {
       uchar	buf[260];
 
-
       if (!gif_eof)
         while (gif_get_block(fp, buf) > 0);
 
@@ -476,17 +488,23 @@ gif_read_lzw(FILE *fp,			/* I - File to read from */
 
     while (code >= clear_code)
     {
+      if (sp >= (stack + sizeof(stack) / sizeof(stack[0])))
+        return (255);
+
       *sp++ = table[1][code];
+
       if (code == table[0][code])
 	return (255);
 
       code = table[0][code];
     }
 
-    *sp++ = firstcode = table[1][code];
-    code  = max_code;
+    if (sp >= (stack + sizeof(stack) / sizeof(stack[0])))
+      return (255);
 
-    if (code < 4096)
+    *sp++ = firstcode = table[1][code];
+
+    if ((code = max_code) < 4096)
     {
       table[0][code] = oldcode;
       table[1][code] = firstcode;
@@ -915,7 +933,7 @@ image_load_bmp(image_t *img,	/* I - Image to load into */
   colors_used      = (int)read_dword(fp);
   read_dword(fp);
 
-  if (img->width <= 0 || img->width > 8192 || img->height <= 0 || img->height > 8192)
+  if (img->width <= 0 || img->width > IMAGE_MAX_DIM || img->height <= 0 || img->height > IMAGE_MAX_DIM || info_size < 0)
     return (-1);
 
   if (info_size > 40)
@@ -927,7 +945,7 @@ image_load_bmp(image_t *img,	/* I - Image to load into */
   // Get colormap...
   if (colors_used == 0 && depth <= 8)
     colors_used = 1 << depth;
-  else if (colors_used > 256)
+  else if (colors_used < 0 || colors_used > 256)
     return (-1);
 
   fread(colormap, (size_t)colors_used, 4, fp);
@@ -1186,7 +1204,7 @@ image_load_bmp(image_t *img,	/* I - Image to load into */
 	    */
 
             if (color < 0)
-	      temp = getc(fp);
+	      temp = getc(fp) & 255;
 	    else
 	      temp = color;
 
@@ -1267,7 +1285,7 @@ image_load_gif(image_t *img,	/* I - Image pointer */
   img->height = (buf[9] << 8) | buf[8];
   ncolors     = 2 << (buf[10] & 0x07);
 
-  if (img->width <= 0 || img->width > 32767 || img->height <= 0 || img->height > 32767)
+  if (img->width <= 0 || img->width > IMAGE_MAX_DIM || img->height <= 0 || img->height > IMAGE_MAX_DIM)
     return (-1);
 
   // If we are writing an encrypted PDF file, bump the use count so we create
@@ -1311,6 +1329,13 @@ image_load_gif(image_t *img,	/* I - Image pointer */
 	      return (-1);
 	  }
 
+          img->width  = (buf[5] << 8) | buf[4];
+          img->height = (buf[7] << 8) | buf[6];
+          img->depth  = gray ? 1 : 3;
+
+	  if (img->width <= 0 || img->width > IMAGE_MAX_DIM || img->height <= 0 || img->height > IMAGE_MAX_DIM)
+	    return (-1);
+
           if (transparent >= 0)
           {
            /*
@@ -1342,13 +1367,6 @@ image_load_gif(image_t *img,	/* I - Image pointer */
             image_need_mask(img);
 	  }
 
-          img->width  = (buf[5] << 8) | buf[4];
-          img->height = (buf[7] << 8) | buf[6];
-          img->depth  = gray ? 1 : 3;
-
-	  if (img->width <= 0 || img->width > 32767 || img->height <= 0 || img->height > 32767)
-	    return (-1);
-
 	  if (!load_data)
 	    return (0);
 
@@ -1432,6 +1450,12 @@ JSAMPROW			row;		/* Sample row pointer */
   img->height = (int)cinfo.output_height;
   img->depth  = (int)cinfo.output_components;
 
+  if (img->width <= 0 || img->width > IMAGE_MAX_DIM || img->height <= 0 || img->height > IMAGE_MAX_DIM)
+  {
+    jpeg_destroy_decompress(&cinfo);
+    return (-1);
+  }
+
   if (!load_data)
   {
     jpeg_destroy_decompress(&cinfo);
@@ -1477,7 +1501,7 @@ image_load_png(image_t *img,	/* I - Image pointer */
   png_structp	pp;		/* PNG read pointer */
   png_infop	info;		/* PNG info pointers */
   int		depth;		/* Input image depth */
-  png_bytep	*rows;		/* PNG row pointers */
+  png_bytep	*rows = NULL;	/* PNG row pointers */
   uchar		*inptr,		/* Input pixels */
 		*outptr;	/* Output pixels */
   int		color_type,	/* PNG color mode */
@@ -1507,19 +1531,20 @@ image_load_png(image_t *img,	/* I - Image pointer */
     return (-1);
   }
 
-  rows = NULL;
-
   if (setjmp(png_jmpbuf(pp)))
   {
     progress_error(HD_ERROR_BAD_FORMAT, "PNG file contains errors!");
 
     png_destroy_read_struct(&pp, &info, NULL);
 
-    if (img != NULL && img->pixels != NULL)
+    if (img != NULL)
+    {
       free(img->pixels);
+      img->pixels = NULL;
+    }
 
-    if (rows != NULL)
-      free(rows);
+    free(rows);
+    rows = NULL;
 
     return (-1);
   }
@@ -1586,6 +1611,12 @@ image_load_png(image_t *img,	/* I - Image pointer */
   img->width  = (int)png_get_image_width(pp, info);
   img->height = (int)png_get_image_height(pp, info);
 
+  if (img->width <= 0 || img->width > IMAGE_MAX_DIM || img->height <= 0 || img->height > IMAGE_MAX_DIM)
+  {
+    png_destroy_read_struct(&pp, &info, NULL);
+    return (-1);
+  }
+
   if (color_type & PNG_COLOR_MASK_ALPHA)
   {
     if ((PSLevel == 0 && PDFVersion >= 14) || PSLevel == 3)
@@ -1616,7 +1647,7 @@ image_load_png(image_t *img,	/* I - Image pointer */
     return (0);
   }
 
-  img->pixels = (uchar *)calloc(1,(size_t)(img->width * img->height * depth));
+  img->pixels = (uchar *)calloc(1, (size_t)(img->width * img->height * depth));
 
  /*
   * Allocate pointers...
@@ -1708,11 +1739,11 @@ image_load_png(image_t *img,	/* I - Image pointer */
   * Free memory and return...
   */
 
-  free(rows);
-
   png_read_end(pp, info);
   png_destroy_read_struct(&pp, &info, NULL);
 
+  free(rows);
+
   return (0);
 }
 #endif // HAVE_LIBPNG
@@ -1783,7 +1814,7 @@ image_set_mask(image_t *img,	/* I - Image to operate on */
 
 
   if (img == NULL || img->mask == NULL || x < 0 || x >= img->width ||
-      y < 0 || y > img->height)
+      y < 0 || y >= img->height)
     return;
 
   if (img->maskscale == 8)
