@@ -7,7 +7,7 @@
  * ZIPC_ONLY_WRITE to compile with just the ZIP writing code.  Otherwise both
  * ZIP reader and writer code is built.
  *
- * Copyright 2017-2019 by Michael R Sweet.
+ * Copyright © 2017-2021 by Michael R Sweet.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -493,8 +493,8 @@ zipcFileGets(zipc_file_t *zf,           /* I - ZIP container file */
              char        *line,         /* I - Line string buffer */
              size_t      linesize)      /* I - Size of buffer */
 {
-  char  ch,                             /* Current character */
-        *lineptr,                       /* Pointer into buffer */
+  int   ch;                             /* Current character */
+  char  *lineptr,                       /* Pointer into buffer */
         *lineend;                       /* Pointer to end of buffer */
 
 
@@ -624,8 +624,8 @@ zipcFileRead(zipc_file_t *zf,           /* I - ZIP container file */
   if (zc->readptr && zc->readptr < zc->readend)
   {
     rbytes = zc->readend - zc->readptr;
-    if (rbytes > bytes)
-      rbytes = bytes;
+    if (rbytes > (ssize_t)bytes)
+      rbytes = (ssize_t)bytes;
 
     memcpy(data, zc->readptr, rbytes);
     zc->readptr += rbytes;
@@ -730,8 +730,8 @@ zipcFileXMLGets(zipc_file_t *zf,        /* I - ZIP container file */
                 char        *fragment,  /* I - Fragment string buffer */
                 size_t      fragsize)   /* I - Size of buffer */
 {
-  char  ch,                             /* Current character */
-        *fragptr,                       /* Pointer into buffer */
+  int   ch;                             /* Current character */
+  char  *fragptr,                       /* Pointer into buffer */
         *fragend;                       /* Pointer to end of buffer */
 
 
@@ -1043,10 +1043,10 @@ zipcOpen(const char *filename,		/* I - Filename of container */
       {
         case ZIPC_LOCAL_HEADER :
             offset            = ftell(zc->fp) - 4;
-            /*version           = */zipc_read_u16(zc);
+            /*version =*/       zipc_read_u16(zc);
             flags             = zipc_read_u16(zc);
             method            = zipc_read_u16(zc);
-            /*modtime           = */zipc_read_u32(zc);
+            /*modtime =*/       zipc_read_u32(zc);
             crc32             = zipc_read_u32(zc);
             compressed_size   = zipc_read_u32(zc);
             uncompressed_size = zipc_read_u32(zc);
@@ -1069,20 +1069,27 @@ zipcOpen(const char *filename,		/* I - Filename of container */
             cfile[cfile_len] = '\0';
 
             if (extra_field_len > 0)
-              fseek(zc->fp, extra_field_len, SEEK_CUR);
+            {
+              if (fseek(zc->fp, extra_field_len, SEEK_CUR) < 0)
+              {
+                fclose(zc->fp);
+                free(zc->files);
+                free(zc);
+                return (NULL);
+              }
+            }
 
             if (zc->num_files >= zc->alloc_files)
             {
               zc->alloc_files += 10;
 
-              if (!zc->files)
-                zf = malloc(zc->alloc_files * sizeof(zipc_file_t));
-              else
-                zf = realloc(zc->files, zc->alloc_files * sizeof(zipc_file_t));
+	      zf = realloc(zc->files, zc->alloc_files * sizeof(zipc_file_t));
 
               if (!zf)
               {
-                zc->error = strerror(errno);
+                fclose(zc->fp);
+                free(zc->files);
+                free(zc);
                 return (NULL);
               }
 
@@ -1104,7 +1111,13 @@ zipcOpen(const char *filename,		/* I - Filename of container */
             zf->offset            = (size_t)offset;
             zf->local_size        = (size_t)(ftell(zc->fp) - offset);
 
-            fseek(zc->fp, compressed_size, SEEK_CUR);
+            if (fseek(zc->fp, compressed_size, SEEK_CUR) < 0)
+            {
+	      fclose(zc->fp);
+	      free(zc->files);
+	      free(zc);
+	      return (NULL);
+            }
             break;
 
        default :
@@ -1191,15 +1204,26 @@ zipcOpenFile(zipc_t     *zc,            /* I - ZIP container */
   {
     if (!strcmp(filename, current->filename))
     {
-      fseek(zc->fp, current->offset + current->local_size, SEEK_SET);
+      if (fseek(zc->fp, current->offset + current->local_size, SEEK_SET) < 0)
+        break;
 
       if (current->method == ZIPC_COMP_DEFLATE)
       {
+        int err;                        /* Error status from inflateInit2 */
+
         zc->stream.zalloc = (alloc_func)0;
         zc->stream.zfree  = (free_func)0;
         zc->stream.opaque = (voidpf)0;
 
-        inflateInit2(&zc->stream, -15);
+        if ((err = inflateInit2(&zc->stream, -15)) < 0)
+        {
+          if (err == Z_STREAM_ERROR)
+            zc->error = strerror(EINVAL);
+          else
+            zc->error = strerror(ENOMEM);
+
+          return (NULL);
+        }
 
         zc->stream.next_in  = (Bytef *)zc->buffer;
         zc->stream.avail_in = 0;
@@ -1393,7 +1417,12 @@ zipc_add_file(zipc_t     *zc,		/* I - ZIP container */
     zc->stream.zfree  = (free_func)0;
     zc->stream.opaque = (voidpf)0;
 
-    deflateInit2(&zc->stream, Z_BEST_COMPRESSION, Z_DEFLATED, -15, 8, Z_DEFAULT_STRATEGY);
+    if (deflateInit2(&zc->stream, Z_BEST_COMPRESSION, Z_DEFLATED, -15, 8, Z_DEFAULT_STRATEGY) < 0)
+    {
+      zc->num_files --;
+      zc->error = strerror(ENOMEM);
+      return (NULL);
+    }
 
     zc->stream.next_out  = (Bytef *)zc->buffer;
     zc->stream.avail_out = sizeof(zc->buffer);
@@ -1664,13 +1693,21 @@ zipc_write_local_trailer(
     * Update the CRC-32, compressed size, and uncompressed size fields...
     */
 
-    fseek(zc->fp, (long)(zf->offset + 14), SEEK_SET);
+    if (fseek(zc->fp, (long)(zf->offset + 14), SEEK_SET) < 0)
+    {
+      zc->error = strerror(errno);
+      return (-1);
+    }
 
     status |= zipc_write_u32(zc, zf->crc32);
     status |= zipc_write_u32(zc, (unsigned)zf->compressed_size);
     status |= zipc_write_u32(zc, (unsigned)zf->uncompressed_size);
 
-    fseek(zc->fp, 0, SEEK_END);
+    if (fseek(zc->fp, 0, SEEK_END) < 0)
+    {
+      zc->error = strerror(errno);
+      return (-1);
+    }
 
     zf->flags &= ZIPC_FLAG_MASK;
   }
